
import { setIntents } from './intents/store';
import { setEntities } from './entities/store';


export const setServiceData = service => {
    return dispatch => {
        dispatch( setEntities( service.entities));
        dispatch( setIntents( service.intents));
    };
}


export const clearService = () => {
  return { type: 'CLEAR_SERVICE', payload : null }
}

export const selectService = ( serviceId) => {
  return { type: 'SELECT_SERVICE', payload : serviceId }
}

export const setServiceWorkflow = ( workflow) => {
  return { type: 'SET_SERVICE_WORKFLOW', payload : workflow }
}

export const addPackage = ( packageId) => {
  return { type: 'ADD_PACKAGE', payload : packageId }
}

export const removePackage = ( packageId) => {
  return { type: 'REMOVE_PACKAGE', payload : packageId }
}

const defaultState  =   {
    service_id : null,
    packages : [],  
    blocks : [],  
    fragments : [],  
    contexts : [],  
    intents : [],  
    entities : [],  
};

export default (state = defaultState, {type, payload}) => {
    switch(type) {
        
        case 'CLEAR_SERVICE':
            return Object.assign( {}, defaultState);
        
        case 'SELECT_SERVICE': 
            return Object.assign( {}, state, { service_id: payload});
            
        case 'SET_SERVICE_WORKFLOW': 
            return Object.assign( {}, state, { 
                packages: payload.packages,
                intents: payload.intents,
                entities: payload.entities,
            });

        case 'ADD_PACKAGE': 
            return Object.assign( {}, state, { 
                packages: [...state.packages, payload],
            });

        case 'REMOVE_PACKAGE': 
            return Object.assign( {}, state, { 
                packages: state.packages.filter( function ( val) {
                    return val.namespace !== payload
                }),
            });

        default: 
            return state;
    }
}
