
import createServiceTemplate from './convoworks-add-service.tmpl.html';
import deleteServiceTemplate from './convoworks-delete-service.tmpl.html';
import ModalInstanceCtrl from './convoworks-add-service.controller';
import ServiceDeleteModalCtrl from './convoworks-delete-service.controller';

ConvoworksMainController.$inject = [ '$log', '$scope', '$uibModal', 'ConvoworksApi'];

export default function ConvoworksMainController( $log, $scope, $uibModal, ConvoworksApi)
{
    $log.debug( 'ConvoworksMainController init');

    // API
    $scope.ready                =   false;
    $scope.availableServices    =   [];
    $scope.filtered             =   [];

    $scope.filter               =   '';

    $scope.createService        =   function()
    {
        $uibModal.open({
            template: createServiceTemplate,
            controller: ModalInstanceCtrl,
            size : 'md',
            resolve: { ConvoworksApi: function() { return ConvoworksApi; }}
        })
    };

    $scope.deleteService        =   function($event, serviceId)
    {
        // $event.preventDefault();
        // $event.stopPropagation();

        var instance = $uibModal.open({
            template: deleteServiceTemplate,
            controller: ServiceDeleteModalCtrl,
            size: 'md',
            resolve: {
                ConvoworksApi: function() { return ConvoworksApi; },
                serviceId: function() { return serviceId; }
            }
        });

        instance.result.then(function (res) {
            $log.log('ConvoworksMainController deleteService modal then res', res);

            if (res.errors && Object.keys(res.errors).length > 0)
            {
                throw new Error('Delete encountered errors ['+JSON.stringify(res.errors, null, 2)+']')
            }

            $scope.ready = false;
            _init();
        })
    }

    $scope.publishedOn = function(service) {
        var published = [];

        angular.forEach(service.versions, function (value, key) {
            if (!published.includes(key)) {
                published.push(_cleanKey(key));
            }
        });

        return published;
    }

    _init();

    // INIT
    function _init()
    {
        ConvoworksApi.getAllServices().then( function( services) {
            $scope.availableServices    =   services;

            $scope.$watch('filter', function() {
                if (!$scope.filter || $scope.filter === '') {
                    $scope.filtered = angular.copy($scope.availableServices);
                }

                $scope.$applyAsync(function() {
                    $scope.filtered = $scope.availableServices.filter(function (service) {
                        return service.name.toLowerCase().includes($scope.filter.toLowerCase());
                    })
                })
            })
        }, function( reason) {
            $log.warn( 'ConvoworksMainController fetching all services failed because of', reason);

            throw new Error( reason.data.message);
        }).finally( function() {
            $scope.ready    =   true;
        })
    }

    function _cleanKey(key) {
        return key.split('_').map(function (word) { return word.charAt(0).toUpperCase() + word.slice(1).toLowerCase(); }).join(' ');
    }
}
