import React, { useEffect, useContext } from 'react';
import { useSelector, useDispatch, useStore } from 'react-redux';
import { IconTrash } from '../../assets/icon_trash.jsx';
import { setTestListItems } from '../../actions/index.jsx';
import { useState } from 'react';
import IntentDetails from './details/intent_details.jsx';

/* Entity editor index page */
function TestList(props) {
	//
	const dispatch = useDispatch();
	const intents = useSelector((state) => state.testListItemsReducer);

	const [ editorState, setEditorState ] = useState('list');
	const [ intent, setIntent ] = useState(null);

	const handleItemRemove = (item, index) => {
		console.log('ITEM_REMOVE: ', item, index);
		let arr = intents;
		arr.splice(index, 1);

		dispatch(setTestListItems(arr));
		setEditorState('list');
	};

	function makeItems(items) {
		return items.map((item, i) => {
			if (item) {
				return (
					<li
						className="item"
						key={i}
						onClick={() => {
							setEditorState('details');
							setIntent({ intent: item, index: i });
						}}
					>
						<div className="item__value--primary font--14--large">
							{item.type ? '' : '@'} {item.name}
						</div>
						<div className="item__buttons">
							<button className="item__remove" onClick={() => handleItemRemove(item, i)}>
								<IconTrash />
							</button>
						</div>
					</li>
				);
			} else {
				return null;
			}
		});
	}

	function makeTestList() {
		return (
			<div className="editor">
				<section className="layout--editor-content">
					<div className="margin--40--large">
						<header className="editor__header margin--24--large">
							<h3 className="no-margin font--18--large color-gray">Test list Items</h3>
						</header>
						<ul>
							{intents ? (
								makeItems(intents)
							) : (
								<div className="color-gray--light">Empty</div>
							)}
						</ul>
					</div>
				</section>
			</div>
		);
	}

	if (editorState === 'list') {
		return makeTestList();
	} else if (editorState === 'details') {
		return <IntentDetails intent={intent.intent} index={intent.index} intents={intents} />;
	} else {
		return null;
	}
}

export {TestList};
