<?php


namespace Convo\Core\Adapters\Viber;


class ViberServicePublisher extends \Convo\Core\Publish\AbstractServicePublisher
{
    /**
     * @var ViberApi
     */
    private $_viberApi;
    public function __construct($logger, \Convo\Core\IAdminUser $user, $serviceId, $viberApi, $serviceDataProvider, $serviceReleaseManager)
    {
        parent::__construct($logger, $user, $serviceId, $serviceDataProvider, $serviceReleaseManager);
        $this->_viberApi = $viberApi;
    }

    /**
     * @inheritDoc
     */
    public function getPlatformId()
    {
        return 'viber';
    }

    /**
     * @inheritDoc
     */
    public function export()
    {
        throw new \Exception( 'Not supported yet');
    }

    public function enable()
    {
        parent::enable();
        $config = $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');
        if (isset( $config[$this->getPlatformId()])) {
            $this->_updateViberChatBot();
        } else {
            throw new \Exception("Missing platform config [" . $this->getPlatformId());
        }
    }

    public function propagate()
    {
        $config = $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');
        if (isset( $config[$this->getPlatformId()])) {
            $this->_updateViberChatBot();
            $this->_recordPropagation();
        } else {
            throw new \Exception("Missing platform config [" . $this->getPlatformId());
        }
    }

    public function getPropagateInfo()
    {
        $this->_logger->info("Accessing propagate info...");
        $data              =   parent::getPropagateInfo();
        $config            =   $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');

        if ( !isset( $config[$this->getPlatformId()])) {
            $this->_logger->debug( 'No platform ['.$this->getPlatformId().'] config in service ['.$this->_serviceId.']. Exiting ... ');
            return $data;
        }

        $this->_logger->info(print_r($config, true) . " Accessing platform config");
        $platform_config   =   $config[$this->getPlatformId()];

        $this->_logger->debug( 'Got auto mode. Checking further ... ');

        $data['allowed'] = true;

        $workflow  =   $this->_convoServiceDataProvider->getServiceData( $this->_user, $this->_serviceId, 'develop');
        $meta      =   $this->_convoServiceDataProvider->getServiceMeta( $this->_user, $this->_serviceId, 'develop');

        if (isset($meta['release_mapping'][$this->getPlatformId()])) {
            $alias     =   $this->_serviceReleaseManager->getDevelopmentAlias( $this->_user, $this->_serviceId, $this->getPlatformId());
            $mapping   =   $meta['release_mapping'][$this->getPlatformId()][$alias];

            if ( !isset( $mapping['time_propagated']) || empty( $mapping['time_propagated'])) {
                $this->_logger->debug( 'Never propagated ');
                $data['available'] = true;
            } else {
                if ( $mapping['time_propagated'] < $platform_config['time_updated']) {
                    $this->_logger->debug( 'Config changed');
                    $data['available'] = true;
                }

                if ( isset( $mapping['time_updated']) && ($mapping['time_propagated'] < $mapping['time_updated'])) {
                    $this->_logger->debug( 'Mapping changed');
                    $data['available'] = true;
                }

                if ( $mapping['time_propagated'] < $workflow['intents_time_updated']) {
                    $this->_logger->debug( 'Intents model changed');
                    $data['available'] = true;
                }
            }
        }

        return $data;
    }

    private function _updateViberChatBot() {
        $config            =   $this->_convoServiceDataProvider->getServicePlatformConfig( $this->_user, $this->_serviceId, 'develop');
        $url = $this->_serviceReleaseManager->getWebhookUrl($this->_user, $this->_serviceId, $this->getPlatformId());
        $this->_viberApi->setupViberApi($this->_user, $this->_serviceId, $config);
        $this->_viberApi->callSetupWebhook($url);
    }

    public function delete(array &$report)
    {
        // TODO: Implement delete() method.
    }
}
