<?php declare(strict_types=1);

namespace Convo\Core;

interface IServiceDataProvider
{

    const DEFAULT_WORKFLOW	=	[
        'service_id' => null,
        'convo_service_version' => \Convo\Core\Factory\ConvoServiceFactory::SERVICE_VERSION,
        'packages' => ['convo-core'],
        'contexts' => [],
        'variables' => [],
        'preview_variables' => [],
        'entities' => [],
        'intents' => [],
        'blocks' => [],
        'fragments' => [],
        'time_updated' => 0,
        'intents_time_updated' => 0,
    ];

    const DEFAULT_META	=	[
        'service_id' => null,
        'name' => null,
        'description' => null,
        'active' => 0,
        'owner' => null,
        'admins' => [],
        'release_mapping' => [],
        'time_updated' => 0,
    ];

    const DEFAULT_RELEASE	=	[
        'service_id' => null,
        'release_id' => null,
        'platform_id' => null,
        'version_id' => null,
        'type' => null,
        'stage' => null,
        'alias' => null,
        'time_created' => 0,
        'time_updated' => 0,
    ];

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param string $versionId
	 * @throws \Convo\Core\DataItemNotFoundException
	 * @throws \Convo\Core\Rest\NotAuthorizedException
	 * @return array
	 */
	public function getServiceData( \Convo\Core\IAdminUser $user, $serviceId, $versionId);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param string $versionId
	 * @throws \Convo\Core\DataItemNotFoundException
	 * @return array
	 */
	public function getServiceMeta( \Convo\Core\IAdminUser $user, $serviceId, $versionId=null);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param array $meta
	 * @param string $versionId
	 * @throws \Convo\Core\DataItemNotFoundException
	 * @return array
	 */
	public function saveServiceMeta( \Convo\Core\IAdminUser $user, $serviceId, $meta, $versionId=null);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @return array
	 */
	public function getAllServices( \Convo\Core\IAdminUser $user);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @return array
	 */
	public function getAllServiceVersions( \Convo\Core\IAdminUser $user, $serviceId);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceName
	 * @param array $workflowData
	 * @return string new service_id
	 */
	public function createNewService( \Convo\Core\IAdminUser $user, $serviceName, $workflowData);


	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param array $data
	 * @return array
	 */
	public function saveServiceData( \Convo\Core\IAdminUser $user, $serviceId, $data);


	public function createRelease( \Convo\Core\IAdminUser $user, $serviceId, $platformId, $type, $stage, $alias, $versionId);


	public function getReleaseData( \Convo\Core\IAdminUser $user, $serviceId, $releaseId);


	public function createServiceVersion( \Convo\Core\IAdminUser $user, $serviceId, $workflow, $config, $versionTag=null);


	public function updateReleaseData( \Convo\Core\IAdminUser $user, $serviceId, $releaseId, $data);


	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param string $versionId
	 * @return array
	 */
	public function getServicePlatformConfig( \Convo\Core\IAdminUser $user, $serviceId, $versionId);

	/**
	 * @param \Convo\Core\IAdminUser $user
	 * @param string $serviceId
	 * @param array $config
	 */
	public function updateServicePlatformConfig( \Convo\Core\IAdminUser $user, $serviceId, $config);
}
