<?php declare(strict_types=1);

namespace Convo\Proto;


use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Psr\Http\Message\ResponseInterface;
use Convo\Core\DataItemNotFoundException;

class AdminAuthMiddleware implements \Psr\Http\Server\MiddlewareInterface
{	
	
	/**
	 * @var \Psr\Log\LoggerInterface
	 */
	private $_logger;
	
	/**
	 * @var \Convo\Core\IAdminUserDataProvider
	 */
	private $_adminUserDataProvider;
	
	public function __construct( \Psr\Log\LoggerInterface $logger, $adminUserDataProvider)
	{
		$this->_logger	=	$logger;
		$this->_adminUserDataProvider	=	$adminUserDataProvider;
	}
	
	public function process( ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
	{
		if ( isset( $_SESSION['username']) && !empty( $_SESSION['username'])) {
		    
		    try {
		        $user	=	$this->_adminUserDataProvider->findUser( $_SESSION['username']);
		        $this->_logger->debug( 'Setting to attributes');
		        $request	=	$request->withAttribute( \Convo\Core\IAdminUser::class, $user);
		    } catch ( DataItemNotFoundException $e) {
		        $this->_logger->warning( $e->getMessage());
		    }
		}
		
		return $handler->handle( $request);
	}
	
	// UTIL
	public function __toString()
	{
		return get_class( $this).'[]';
	}
}