<?php declare(strict_types=1);

namespace Convo\Proto\Pckg\Elements;

class RemoveFromCart extends \Convo\Core\Workflow\AbstractWorkflowComponent implements \Convo\Core\Workflow\IConversationElement
{
    private $_input = [];
    
    public function __construct($properties) {
        
        parent::__construct( $properties);
        $this->_input = $properties['input'];
    }
    
    public function read( \Convo\Core\Workflow\IConvoRequest $request, \Convo\Core\Workflow\IConvoResponse $response)
    {
        if ( !is_a($request, '\Convo\Core\Adapters\Alexa\AmazonCommandRequest') &&
            !is_a($request, '\Convo\Core\Adapters\Google\Dialogflow\DialogflowCommandRequest'))
        {
            throw new \Exception('Request cannot support touch capabilities.');
        }
        
        $this->_logger->debug( 'Raw input ['.$this->_input.']');
        
        /** @var \Convo\Core\Adapters\Alexa\AmazonCommandRequest|\Convo\Core\Adapters\Google\Dialogflow\DialogflowCommandRequest $request */
        $index = $request->getSelectedOption();
        
        $this->_logger->debug( 'selected index ['.$index.']');
        
        $params = $this->getService()->getServiceParams(\Convo\Core\Params\IServiceParamsScope::SCOPE_TYPE_SESSION);
        
        $cart = $params->getServiceParam( $this->_input)['cart'];
        
        array_splice( $cart, intval($index), 1);
        
        $params->setServiceParam( $this->_input, [
            'count' => count( $cart),
            'cart' => $cart,
        ]);
        
        $this->_logger->debug( 'Cart products ['.print_r( $params->getServiceParam( $this->_input), true).']');
        
    }
    
    // UTIL
    public function __toString()
    {
        return parent::__toString().'['.$this->_input.']';
    }
    
}

?>
