<?php declare(strict_types=1);

namespace Convo\Proto\Pckg;

use Convo\Core\Factory\AbstractPackageDefinition;
use Convo\Core\Factory\ComponentDefinition;
use Convo\Core\Factory\IComponentFactory;

class ProtoPackageDefinition extends AbstractPackageDefinition
{
    const NAMESPACE = 'convo-proto';

    /**
     * @var \Convo\Proto\ServiceUserDao
     */
    private $_serviceUserDao;

    public function __construct(\Psr\Log\LoggerInterface $logger, \Convo\Proto\ServiceUserDao $serviceUserDao)
    {
        $this->_serviceUserDao = $serviceUserDao;

        parent::__construct($logger, self::NAMESPACE, __DIR__);
    }

    /**
	 * {@inheritDoc}
	 * @see \Convo\Core\Factory\AbstractPackageDefinition::_initDefintions()
	 */
    protected function _initDefintions()
    {
        return [
			new ComponentDefinition(
				$this->getNamespace(),
                '\Convo\Proto\Pckg\Elements\GetAuthUserElement',
                'Init current auth user',
                'Initialize the currently authenticated user.',
                [
                    'name' => [
                        'editor_type' => 'text',
                        'editor_properties' => [],
                        'defaultValue' => 'user',
                        'name' => 'Name',
                        'description' => 'Name under which to store the loaded user object in the context',
                        'valueType' => 'string'
                    ],
                    'prompt_for_linking' => [
                        'editor_type' => 'boolean',
                        'editor_properties' => [],
                        'defaultValue' => false,
                        'name' => 'Prompt for linking',
                        'description' => 'Prompt the user to link their account if an authenticated user could not be loaded.',
                        'valueType' => 'boolean'
                    ],
                    '_preview_angular' => [
                        'type' => 'html',
                        'template' => '<div class="code">' .
                            'Load user and set it as <span class="statement"><b>{{ component.properties.name }}</b></span>' .
                            '</div>'
                    ],
                    '_workflow' => 'read',
                    '_factory' => new class ($this->_serviceUserDao) implements IComponentFactory
                    {
                        private $_serviceUserDao;

                        public function __construct(\Convo\Proto\ServiceUserDao $serviceUserDao)
                        {
                            $this->_serviceUserDao = $serviceUserDao;
                        }

                        public function createComponent($properties, $service)
                        {
                            return  new \Convo\Proto\Pckg\Elements\GetAuthUserElement($properties, $this->_serviceUserDao);
                        }
                    }
                ]
			),
			new \Convo\Core\Factory\ComponentDefinition(
			    $this->getNamespace(),
			    '\Convo\Proto\Pckg\Elements\CategorySearch',
			    'Category Search',
			    'category',
			    array(
			        'json_path' => array(
			            'editor_type' => 'ssml',
			            'editor_properties' => array(),
			            'defaultValue' => '',
			            'name' => 'path to folder with jsons',
			            'valueType' => 'string'
			        ),
			        'search' => array(
			            'editor_type' => 'ssml',
			            'editor_properties' => array(),
			            'defaultValue' => '',
			            'name' => 'search string',
			            'valueType' => 'string'
			        ),
			        'result' => array(
			            'editor_type' => 'ssml',
			            'editor_properties' => array(),
			            'defaultValue' => '',
			            'name' => 'result variable name',
			            'valueType' => 'string'
			        ),
			        '_preview_angular' => array(
			            'type' => 'html',
			            'template' => '<div class="we-say"><b>Category Search: {{component.properties.json_path}}</b></div>'
			        ),
			        '_workflow' => 'read',
			    )
			    ),
			    new \Convo\Core\Factory\ComponentDefinition(
			        $this->getNamespace(),
			        '\Convo\Proto\Pckg\Elements\ProductSearch',
			        'Product Search',
			        'product',
			        array(
			            'json_path' => array(
			                'editor_type' => 'ssml',
			                'editor_properties' => array(),
			                'defaultValue' => '',
			                'name' => 'path to folder with jsons',
			                'valueType' => 'string'
			            ),
			            'search' => array(
			                'editor_type' => 'ssml',
			                'editor_properties' => array(),
			                'defaultValue' => '',
			                'name' => 'search string',
			                'valueType' => 'string'
			            ),
			            'result' => array(
			                'editor_type' => 'ssml',
			                'editor_properties' => array(),
			                'defaultValue' => '',
			                'name' => 'result variable name',
			                'valueType' => 'string'
			            ),
			            'page_size' => array(
			                'editor_type' => 'ssml',
			                'editor_properties' => array(),
			                'defaultValue' => '',
			                'name' => 'page size for pagination, default 3',
			                'valueType' => 'string'
			            ),
			            'page_no' => array(
			                'editor_type' => 'ssml',
			                'editor_properties' => array(),
			                'defaultValue' => '',
			                'name' => 'current page for pagination',
			                'valueType' => 'string'
			            ),
			            '_preview_angular' => array(
			                'type' => 'html',
			                'template' => '<div class="we-say"><b>Product search: {{component.properties.json_path}}</b></div>'
			            ),
			            '_workflow' => 'read',
			        )
			        ),
			        new \Convo\Core\Factory\ComponentDefinition(
			            $this->getNamespace(),
			            '\Convo\Proto\Pckg\Elements\AddToCart',
			            'Add to cart',
			            'cart',
			            array(
			                'input' => array(
			                    'editor_type' => 'ssml',
			                    'editor_properties' => array(),
			                    'defaultValue' => '',
			                    'name' => 'selected product',
			                    'valueType' => 'array'
			                ),
			                'result' => array(
			                    'editor_type' => 'ssml',
			                    'editor_properties' => array(),
			                    'defaultValue' => '',
			                    'name' => 'result variable name',
			                    'valueType' => 'string'
			                ),
			                '_preview_angular' => array(
			                    'type' => 'html',
			                    'template' => '<div class="we-say"><b>Add To Cart: {{component.properties.input}}</b></div>'
			                ),
			                '_workflow' => 'read',
			            )
			            ),
			            new \Convo\Core\Factory\ComponentDefinition(
			                $this->getNamespace(),
			                '\Convo\Proto\Pckg\Elements\RemoveFromCart',
			                'Remove from cart',
			                'cart',
			                array(
			                    'input' => array(
			                        'editor_type' => 'ssml',
			                        'editor_properties' => array(),
			                        'defaultValue' => '',
			                        'name' => 'cart variable',
			                        'valueType' => 'array'
			                    ),
			                    '_preview_angular' => array(
			                        'type' => 'html',
			                        'template' => '<div class="we-say"><b>Remove From Cart: {{component.properties.input}}</b></div>'
			                    ),
			                    '_workflow' => 'read',
			                )
			                )
        ];
    }
}
