const path = require('path');

var pluginProvide = require('./webpack/plugin-provide');
var pluginCopy = require('./webpack/plugin-copy');
var pluginHtml = require('./webpack/plugin-html');
var loadEnv = require('./webpack/load-env');
var optimization = require('./webpack/optimization');
var rules = require('./webpack/module-rules');


module.exports = function ( env) {

    var config  =   loadEnv( env.ENV);

    if ( env.ENV === 'dev' || env.ENV === 'local') {
        var configFilename  =   '__config.php';
    } else {
        var configFilename  =   '__config.'+env.ENV+'.php';
    }

    return {
        devtool: 'cheap-module-eval-source-map',
        mode: config['ENV_MODE'],
        entry: {
            main: path.resolve('app/proto', 'app.js'),
        },
        output: {
            path: path.resolve(__dirname, 'dist/www'),
                publicPath: '/',
                filename: '[name].[hash].js',
                chunkFilename: '[name].[hash].js',
        },
        optimization: optimization,
        module: {
            rules: rules,
        },
        devServer: {
            historyApiFallback: true,
            host: '0.0.0.0',
            hot: true
        },
        resolve: {
            alias: {
                'jquery-ui': 'jquery-ui-dist/jquery-ui.js'
            }
        },
        plugins: [
            pluginCopy( configFilename),
            pluginProvide(),
            pluginHtml( config, './app/proto/index.ejs', 'index.html'),
        ],
    };
};
